#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
################################################################################
CaseLevel         CONFORMANCE
CaseAttribute     AUTO
CaseVerboseLevel  DEFAULT
set reportfile    report.csv

#
# test case Name, category, description, GUID...
#
CaseGuid        8ce002cf-24f5-4a0c-a984-2ee5711aab43
CaseName        TransmitReceive.Conf2.Case1
CaseCategory    DHCP4
CaseDescription {This case is to test the Conformance - EFI_NOT_READY          \
                -- Previous call not end}

################################################################################

Include DHCP4/include/Dhcp4.inc.tcl

proc CleanUpEutEnvironment {} {
  Dhcp4ServiceBinding->DestroyChild "@R_Handle, &@R_Status"
  GetAck

  EndScope _DHCP4_TRANSMITRECEIVE_CONF2
  EndLog
}

#
# Begin log ...
#
BeginLog

#
# BeginScope
#
BeginScope _DHCP4_TRANSMITRECEIVE_CONF2

#
# Parameter Definition
# R_ represents "Remote EFI Side Parameter"
# L_ represents "Local ENTS Side Parameter"
#
UINTN                                 R_Status
UINTN                                 R_Handle
                                      
UINTN                                 R_Context
EFI_DHCP4_TRANSMIT_RECEIVE_TOKEN      R_Token
EFI_DHCP4_LISTEN_POINT                R_ListenPoint
EFI_DHCP4_PACKET                      R_Packet
EFI_DHCP4_TRANSMIT_RECEIVE_TOKEN      R_Token2
                                      
#
# Call [DHCP4SBP] -> CreateChild to create child.
#
Dhcp4ServiceBinding->CreateChild {&@R_Handle, &@R_Status}
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Dhcp4SBP.CreateChild - Create Child 1"                        \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"
SetVar     [subst $ENTS_CUR_CHILD]  @R_Handle


BS->CreateEvent "$EVT_NOTIFY_SIGNAL, $EFI_TPL_CALLBACK, 1, &@R_Context,        \
                &@R_Token.CompletionEvent, &@R_Status"
GetAck
set assert    [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid  "BS.CreateEvent."               \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Call [DHCP4] -> TransmitReceive with the following parameters
# o TimeoutValue: 		10 seconds
# o CompletionEvent:	A valid Event
#
SetIpv4Address  R_ListenPoint.ListenAddress   "192.168.1.21"
SetIpv4Address  R_ListenPoint.SubnetMask      "255.255.255.0"
SetVar          R_ListenPoint.ListenPort      68

SetVar  R_Packet.Size                     [Sizeof EFI_DHCP4_PACKET]
SetVar  R_Packet.Length                   [expr [Sizeof EFI_DHCP4_HEADER] + 5]
SetVar  R_Packet.Dhcp4.Magik              0x63538263
SetVar  R_Packet.Dhcp4.Header.OpCode      1
SetVar  R_Packet.Dhcp4.Header.HwType      1
SetVar  R_Packet.Dhcp4.Header.HwAddrLen   6
SetVar  R_Packet.Dhcp4.Header.Xid         0xe139e46c
SetIpv4Address  R_Packet.Dhcp4.Header.ClientAddr    "192.168.11.21"
SetVar  R_Packet.Dhcp4.Header.ClientHwAddr  { 0x00 0x0d 0x60 0x36 0x87 0x49}
SetVar  R_Packet.Dhcp4.Option               {0xff}

SetVar  R_Token.Status                    0
SetIpv4Address  R_Token.RemoteAddress     "192.168.1.1" 
SetVar  R_Token.RemotePort                67
SetVar  R_Token.ListenPointCount          1
SetVar  R_Token.ListenPoints              &@R_ListenPoint
SetVar  R_Token.TimeoutValue              10;       #TimeoutValue is 10s
SetVar  R_Token.Packet                    &@R_Packet

Dhcp4->TransmitReceive "&@R_Token, &@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Dhcp4.TransmitReceive - Previous Call Not End - 1nd Call"     \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Call [DHCP4] -> TransmitReceive before TimeoutValue expires.
#
SetVar  R_Token2                          @R_Token
SetVar  R_Token2.CompletionEvent          0

#
# Check Point: Call Dhcp4.TransmitReceive while while the previous call to this 
#              function has not finished yet.
#
Dhcp4->TransmitReceive "&@R_Token2, &@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_NOT_READY]
RecordAssertion $assert $Dhcp4TransmitReceiveConf2AssertionGuid001             \
                "Dhcp4.TransmitReceive - Previous Call Not End - 2nd Call"     \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_NOT_READY"

Stall   10

#
# Close Event.
#
BS->CloseEvent "@R_Token.CompletionEvent, &@R_Status"
GetAck
set assert    [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "BS.CloseEvent."                                               \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Clean up the environment on EUT side.
#
CleanUpEutEnvironment
