#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
################################################################################
CaseLevel         CONFORMANCE
CaseAttribute     AUTO
CaseVerboseLevel  DEFAULT
set reportfile    report.csv

#
# test case Name, category, description, GUID...
#
CaseGuid        AEA3C79D-6A5E-4919-85F9-6E864D761FD8
CaseName        Receive.Conf2.Case1
CaseCategory    MNP
CaseDescription {Test Receive conformance of MNP - Call MNP.Receive() when the \
	               child has not been started. The return status should be       \
	               EFI_NOT_STARTED.}
################################################################################

Include MNP/include/Mnp.inc.tcl

#
# Begin log ...
#
BeginLog
BeginScope  _MNP_RECEIVE_CONFORMANCE1_CASE1_

#
# Parameter Definition
# R_ represents "Remote EFI Side Parameter"
# L_ represents "Local OS Side Parameter"
#
UINTN                            R_Status
UINTN                            R_Handle
EFI_MANAGED_NETWORK_CONFIG_DATA  R_MnpConfData
EFI_MANAGED_NETWORK_COMPLETION_TOKEN  R_Token
UINTN                                 R_Context
#
# Create child R_Handle
#
MnpServiceBinding->CreateChild {&@R_Handle, &@R_Status}
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mnp.Receive - Conformance - Create"                           \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"
SetVar          [subst $ENTS_CUR_CHILD]  @R_Handle

#
# Call Receive when the child not start
#
BS->CreateEvent  "$EVT_NOTIFY_SIGNAL, $EFI_TPL_CALLBACK, 1, &@R_Context,         \
                &@R_Token.Event, &@R_Status"
GetAck
Mnp->Receive "&@R_Token, &@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_NOT_STARTED]
RecordAssertion $assert $MnpReceiveConf2AssertionGuid001                       \
                "Mnp.Receive - Conformance - child not start"                  \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_NOT_STARTED"

#
# Destroy child R_Handle
#
MnpServiceBinding->DestroyChild {@R_Handle, &@R_Status}
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Mnp.Transmit - Basic Func - Dst specified"                    \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

EndScope  _MNP_RECEIVE_CONFORMANCE1_CASE1_

#
# End log
#
EndLog
