#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
if {[info exist UDP_INC_TCL] == 1} {
  return
}

set UDP_INC_TCL 1

Include EfiUtil/EfiInclude.tcl
Include Include/EntsProtocolTest.inc.tcl
Include Include/EfiBootService.inc.tcl
Include MNP/Include/Mnp.inc.tcl
Include MNP/Include/Mnp.proc.tcl
Include SNP/Include/Snp.inc.tcl
Include IP4/Include/Ip4.inc.tcl
Include UDP4/Include/Udp4.proc.tcl

Include Include/GenericAssertionGuid.tcl
Include UDP4/Include/Udp4AssertionGuid.tcl


Func Udp4ServiceBinding->CreateChild {
  POINTER
  POINTER
}

Func Udp4ServiceBinding->DestroyChild {
  UINTN
  POINTER
}
#*************************************************
#      EFI_UDP4_FRAGMENT_DATA                    *
#*************************************************
Struct EFI_UDP4_FRAGMENT_DATA {
  UINT32        FragmentLength;
  POINTER       FragmentBuffer;
};

#*************************************************
#      EFI_UDP4_SESSION_DATA                     *
#*************************************************
Struct EFI_UDP4_SESSION_DATA {
  EFI_IPv4_ADDRESS   SourceAddress;
  UINT16             SourcePort;
  EFI_IPv4_ADDRESS   DestinationAddress;
  UINT16             DestinationPort;
};

#************************************************
#           EFI_UDP4_HEADER                                            *
#************************************************
Pack 1
Struct EFI_UDP4_HEADER {
  UINT16     SourcePortNum;
  UINT16     DestinationPortNum;
  UINT16     UdpLength;
  UINT16     UdpCheckSum;
};
Pack

#************************************************
#      EFI_UDP4_CONFIG_DATA                     *
#************************************************
Struct EFI_UDP4_CONFIG_DATA {
  BOOLEAN               AcceptBroadcast;
  BOOLEAN               AcceptPromiscuous;
  BOOLEAN               AcceptAnyPort;
  BOOLEAN               AllowDuplicatePort;
  UINT8                 TypeOfService;
  UINT8                 TimeToLive;
  BOOLEAN               DoNotFragment;
  UINT32                ReceiveTimeout;
  UINT32                TransmitTimeout;
  BOOLEAN               UseDefaultAddress;
  EFI_IPv4_ADDRESS      StationAddress;
  EFI_IPv4_ADDRESS      SubnetMask;
  UINT16                StationPort;
  EFI_IPv4_ADDRESS      RemoteAddress;
  UINT16                RemotePort;
}

#*******************************************************
#               EFI_UDP4_TRANSMIT_DATA                 *
#*******************************************************
Struct EFI_UDP4_TRANSMIT_DATA {
  POINTER                    UdpSessionData;
  POINTER                    GatewayAddress;
  UINT32                     DataLength;
  UINT32                     FragmentCount;
  EFI_UDP4_FRAGMENT_DATA     FragmentTable;
} ;

Struct EFI_UDP4_TRANSMIT_DATA_P4 {
  POINTER                    UdpSessionData;
  POINTER                    GatewayAddress;
  UINT32                     DataLength;
  UINT32                     FragmentCount;
  EFI_UDP4_FRAGMENT_DATA     FragmentTable(4);
} ;

#*******************************************************
#           EFI_UDP4_RECEIVE_DATA                      *
#*******************************************************
Struct EFI_UDP4_RECEIVE_DATA {
  EFI_TIME                  TimeStamp;
  POINTER                   RecycleSignal;
  EFI_UDP4_SESSION_DATA     UdpSession;
  UINT32                    DataLength;
  UINT32                    FragmentCount;
  EFI_UDP4_FRAGMENT_DATA    FragmentTable;
} ;

Struct EFI_UDP4_RECEIVE_DATA_P4 {
  EFI_TIME                  TimeStamp;
  POINTER                   RecycleSignal;
  EFI_UDP4_SESSION_DATA     UdpSession;
  UINT32                    DataLength;
  UINT32                    FragmentCount;
  EFI_UDP4_FRAGMENT_DATA    FragmentTable(4);
} ;

#*******************************************************
#           EFI_UDP4_COMPLETION_TOKEN                  *
#*******************************************************
Union EFI_UDP4_COMPLETION_TOKEN_PACKET {
    POINTER               RxData;
    POINTER               TxData;
} ;


Struct EFI_UDP4_COMPLETION_TOKEN {
  UINTN                                 Event;
  UINTN                                 Status;
  POINTER                               Packet;
}


Func Udp4->GetModeData {
  POINTER
  POINTER
  POINTER
  POINTER
  POINTER
}
Func Udp4->Configure {
  POINTER
  POINTER
}
Func Udp4->Groups {
  BOOLEAN
  POINTER
  POINTER
}
Func Udp4->Routes {
  BOOLEAN
  POINTER
  POINTER
  POINTER
  POINTER
}
Func Udp4->Poll {
  POINTER
}
Func Udp4->Receive {
  POINTER
  POINTER
}
Func Udp4->Transmit {
  POINTER
  POINTER
}
Func Udp4->Cancel {
  POINTER
  POINTER
}

Func Udp4->SignalRecycleEvent {
  POINTER
  POINTER
}

