/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsRivlEndian.c
    
Abstract:

    Implementation of RIVL endian related functions

--*/

#include "EmsRivlMain.h"
#include "EmsRivlEndian.h"

BOOLEAN RivlLittleEndian = TRUE;

UINT64  Lak;

UINT16
RivlSwap16 (
  UINT16  N
  )
/*++

Routine Description:

  Swap the bytes of a INT16

Arguments:

  N - The number should be swapped

Returns:

  The result

--*/
{
  UINT16  M;
  UINT8   *B;
  B = (UINT8 *) &N;
  if (RivlLittleEndian) {
    M = (UINT16) (B[0] << 8 | B[1]);
  } else {
    M = N;
  }

  return M;
}

UINT32
RivlSwap32 (
  UINT32  N
  )
/*++

Routine Description:

  Swap the bytes of a INT32

Arguments:

  N - The number should be swapped

Returns:

  The result

--*/
{
  UINT32  M;
  UINT8   *B;
  B = (UINT8 *) &N;

  if (RivlLittleEndian) {
    M = (UINT32) (B[0] << 24 | B[1] << 16 | B[2] << 8 | B[3]);
  } else {
    M = N;
  }

  return M;
}

UINT64
RivlSwap64 (
  UINT64  N
  )
/*++

Routine Description:

  Swap the bytes of a INT64

Arguments:

  N - The number should be swapped

Returns:

  The result

--*/
{
  union {
    UINT64  N;
    UINT8   B[8];
  } U;

  UINT8 T;
  U.N = N;

  if (RivlLittleEndian) {

    T       = U.B[0];
    U.B[0]  = U.B[7];
    U.B[7]  = T;

    T       = U.B[1];
    U.B[1]  = U.B[6];
    U.B[6]  = T;

    T       = U.B[2];
    U.B[2]  = U.B[5];
    U.B[5]  = T;

    T       = U.B[3];
    U.B[3]  = U.B[4];
    U.B[4]  = T;
  }

  return U.N;
}
