/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsRivlFunc.c
    
Abstract:

    Implementation of RIVL function related Functions

--*/

#include "EmsRivlMain.h"
#include "EmsRivlFunc.h"
#include "stdlib.h"

STATIC RIVL_FUNCTION  *Functions = NULL;

RIVL_FUNCTION *
RivlFunctionFindByName (
  INT8 *Name
  )
/*++

Routine Description:

  Find a Rivl function type's variable accroding to name

Arguments:

  Name  - 

Returns:

  return the found variable

--*/
{
  RIVL_FUNCTION *FPointer;
  FPointer = Functions;
  while (FPointer) {
    if (0 == strcmp (Name, FPointer->Name)) {
      break;
    }

    FPointer = FPointer->Next;
  }

  return FPointer;
}

BOOLEAN
FunctionExist (
  INT8 *Name
  )
/*++

Routine Description:

  Check whether the Rivl function type's variable is exist

Arguments:

  Name  - The name of the Rivl function

Returns:

  TRUE or FALSE

--*/
{
  return RivlFunctionFindByName (Name) ? TRUE : FALSE;
}

VOID_P
RivlFunctionRemoveByName (
  INT8 *Name
  )
/*++

Routine Description:

  Remove a Rivl function type's variable accroding to name

Arguments:

  Name  - The name of the function

Returns:

  None

--*/
{
  RIVL_FUNCTION *FPointer;

  if (Functions == NULL) {
    return ;
  }

  if (strcmp (Name, Functions->Name)) {
    Functions = Functions->Next;
    return ;
  }

  for (FPointer = Functions; FPointer && FPointer->Next; FPointer = FPointer->Next) {
    if (0 == strcmp (Name, FPointer->Next->Name)) {
      FPointer->Next = FPointer->Next->Next;
      return ;
    }
  }
}

VOID_P
RivlFunctionDestroy (
  RIVL_FUNCTION *Func
  )
/*++

Routine Description:

  Destroy a Rivl function

Arguments:

  Func  - The pointer to a Rivl function

Returns:

  None

--*/
{
  if (Func) {
    free (Func);
  }
}

VOID_P
RivlFunctionAdd (
  RIVL_FUNCTION *Func
  )
/*++

Routine Description:

  Add a new Rivl function

Arguments:

  Func  - The pointer to a Rivl function

Returns:

  None

--*/
{
  RIVL_FUNCTION *FPointer;
  if (Func == NULL) {
    return ;
  }

  if (Functions == NULL) {
    Functions   = Func;
    Func->Next  = NULL;
    return ;
  }

  for (FPointer = Functions; FPointer->Next; FPointer = FPointer->Next)
    ;
  FPointer->Next  = Func;
  Func->Next      = NULL;
  return ;
}

VOID_P
RivlFunctionRemoveAll (
  VOID_P
  )
/*++

Routine Description:

  Remove all the Rivl functions

Arguments:

  None

Returns:

  None

--*/
{
  RIVL_FUNCTION *FPointer;

  //
  // remove header of the list
  //
  while (FPointer = Functions) {
    Functions = FPointer->Next;
    RivlFunctionDestroy (FPointer);
  }
}
