/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsRivlGetAck.c
    
Abstract:

    Implementation of RIVL GetAck(TCL command)

--*/

#include <stdlib.h>
#include "EmsRivlType.h"
#include "EmsRivlMain.h"
#include "EmsRpcMain.h"

INT32
TclGetAck (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "GetAck" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter.
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
{
  INT8    Message[MAX_MESSAGE_LEN];
  INT8    ErrorBuff[MAX_ERRBUFF_LEN];
  INT8    TempStr[10];
  INT32   Timeout;
  INT32   Length;
  BOOLEAN Pass;
  INT8    *Out;
  INT8    *Log;

  if (Argc > 1) {
    Timeout = atoi (Argv[1]);
  } else {
    Timeout = -1;
  }

  Timeout = -1;

  //
  // memset  (message, 0, MAX_MESSAGE_LEN);
  //
  Message[0]  = 0;
  Length      = RpcRecvMessage (Timeout, MAX_MESSAGE_LEN, Message);
  if ((FALSE == ParseAckMessage (Length, Message, &Pass, &Out, &Log)) /*||
      (false == Pass)*/ ) {
    sprintf (ErrorBuff, "GetAck failed!");
    goto ErrorExit;
  }
  //
  // Format of GetAck should be
  //      GetAck [timeout [Pass [Out [log]]]]
  //
  switch (Argc) {
  case 5:
    Tcl_SetVar (Interp, Argv[4], Log, TCL_GLOBAL_ONLY);

  case 4:
    Tcl_SetVar (Interp, Argv[3], Out, TCL_GLOBAL_ONLY);

  case 3:
    if (Pass == TRUE) {
      strcpy (TempStr, "true");
      Tcl_SetVar (Interp, Argv[2], TempStr, TCL_GLOBAL_ONLY);
    } else {
      strcpy (TempStr, "false");
      Tcl_SetVar (Interp, Argv[2], TempStr, TCL_GLOBAL_ONLY);
    }
    break;

  case 2:
    break;

  case 1:
    break;

  default:
    sprintf (ErrorBuff, "Contax of GetAck error");
    goto ErrorExit;
  }

  if (Pass != TRUE) {
    sprintf (ErrorBuff, "GetAck failed!");
    goto ErrorExit;
  }

  return TCL_OK;

ErrorExit:
  Tcl_AppendResult (Interp, ErrorBuff, (INT8 *) NULL);

  return TCL_ERROR;
}
