/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsRpcTarget.c
    
Abstract:

    Implementation of RPC target related Functions

--*/

#include "EmsTypes.h"
#include "EmsMain.h"
#include "EmsUtilityString.h"
#include "EmsRpcTarget.h"

STATIC TARGET_T *Targets = NULL;

VOID_P
RpcTargetAddByMac (
  INT8      *Mac
  )
/*++

Routine Description:

  Add a target identified by MAC address

Arguments:

  Mac - The MAC address

Returns:

  None

--*/
{
  TARGET_T  *Node;
  TARGET_T  *TargetPointer;

  if (Mac == NULL) {
    return ;

  }

  Node = malloc (sizeof (TARGET_T));
  if (Node == NULL) {
    return ;
  }

  memcpy (Node->Mac, Mac, 6);

  if (Targets == NULL) {
    Targets     = Node;
    Node->Next  = NULL;
    return ;
  }

  for (TargetPointer = Targets; TargetPointer->Next; TargetPointer = TargetPointer->Next)
    ;

  TargetPointer->Next = Node;
  Node->Next          = NULL;
  return ;
}

TARGET_T *
RpcTargetFindByMac (
  INT8            *Mac
  )
/*++

Routine Description:

  Find the taget by MAC address

Arguments:

  Mac - MAC address

Returns:

  The target found

--*/
{
  TARGET_T  *TargetPointer;

  TargetPointer = Targets;
  while (TargetPointer) {
    if (0 == memcmp (Mac, TargetPointer->Mac, 6)) {
      break;
    }

    TargetPointer = TargetPointer->Next;
  }

  return TargetPointer;
}

VOID_P
RpcTargetRemoveAll (
  VOID_P
  )
/*++

Routine Description:

  Remove all the target

Arguments:

  None.

Returns:

  None

--*/
{
  TARGET_T  *TargetPointer;

  TARGET_T  *TargetPointer1;

  //
  // remove header of the list
  //
  TargetPointer = Targets;
  while (TargetPointer != NULL) {
    TargetPointer1 = TargetPointer->Next;
    free (TargetPointer);
    TargetPointer = TargetPointer1;
  }

  Targets = NULL;
}
//
// dump all Targets to buffer
//
VOID_P
RpcTargetDumpAll (
  INT8        *Buff
  )
/*++

Routine Description:

  Dump all the targets

Arguments:

  Buff  - The Buffer to contain targets

Returns:

  None

--*/
{
  TARGET_T  *TargetPointer;
  INT8      Str[18];

  TargetPointer = Targets;
  if (NULL == TargetPointer) {
    return ;
  }

  Buff[0] = '\0';
  while (TargetPointer) {
    MacToAsciiString (Str, TargetPointer->Mac);
    strcat (Buff, Str);
    TargetPointer = TargetPointer->Next;
  }
}
