/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsEftp.h
    
Abstract:

    Header files for EFTP implementation.

--*/

#ifndef __EMS_EFTP_H__
#define __EMS_EFTP_H__

#include <EmsTypes.h>
#include <EmsTclInit.h>
#include <EmsNet.h>

#define EFTP_TYPE         0x8888
#define MAX_EFTP_BUF_LEN  1500
#define SRC_MAC_OFFSET    0
#define DES_MAC_OFFSET    6
#define TYPE_OFFSET       12
#define LENGTH_OFFSET     14
#define CHECK_SUM_OFFSET  16
#define OPCODE_OFFSET     18
#define BLKNO_OFFSET      20
#define DATA_OFFSET       22
#define FILE_NAME_OFFSET  DATA_OFFSET
#define MAX_MAC_LEN       6
#define MAX_FILENAME_LEN  128
#define MAX_FILEDATA_LEN  1024
#define EFTP_HEADER_LEN   8
#define MAX_TIMEOUT_COUNT 2
#define MAX_TIMEOUT_VALUE 2000
#define FILE_BLK_SIZE     1024
#define EFTP_CRC_LEN      4
#define MAX_SESSION_NUM   6

typedef struct EmsPktStruct {
  struct EmsPktStruct *Next;
  UINT32              Length;
  INT8                Buffer[MAX_EFTP_BUF_LEN];
} EmsEftpPkt;

typedef struct EmsEftpRequestStruct {
  struct EmsEftpRequestStruct *Next;
  HANDLE                      WaitEvent;
} EmsEftpRequest;

STATIC Tcl_CmdProc  TclEftpStart;

VOID
EftpTclInit (
  IN Tcl_Interp *Interp
  )
/*++

Routine Description:

  EMS Eftp related TCL initialization.

Arguments:

  Interp      - TCL intepreter.

Returns:

  None

--*/
;

VOID
EmsEftpModEnable (
  VOID
  )
/*++

Routine Description:

  Enable the EMS Eftp utility

Arguments:

  None

Returns:

  None

--*/
;

#define EFTP_ERROR_MSG  printf
#define EFTP_DEBUG_MSG  printf

#endif
