/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsPktRecvAssertion.h
    
Abstract:

    

--*/

#ifndef __EMS_RECV_ASSERTION_H__
#define __EMS_RECV_ASSERTION_H__

#define PRINT_MAC_STRING_LEN    35
#define PRINT_TYPE_STRING_LEN   10
#define PRINT_GUID_STRING_LEN   50
#define PRINT_RESULT_STRING_LEN 15
#define PRINT_DISC_STRING_LEN   1400

//
// Recv port
//
#define ASSERTION_LOCAL_PORT  1514
#define ASSERTION_REMOTE_PORT 1514

typedef struct _ASSERTION_PACKAGE {
  UINT8   *ReceiveBuff;       // Buffer the lasted packet received
  UINT32  ReceiveLen;
  UINT32  Received;           // Received
  pcap_t  *Capturer;          // Pcap Capturer, for receiving remote packet
  UINT8   CaptureFilter[100]; // Packet filter
} ASSERTION_PACKAGE;

STATIC Tcl_CmdProc  EmsRecvAssertionPkt;

VOID
EmsRecvAssertionThread (
  VOID* Parameter
  )
/*++

Routine Description:

  The thread routine of receiving assertion message

Arguments:

  Parameter - The parameter of the thread routine

Returns:

  None

--*/
;

VOID_P
EmsRecvedAssertionPkt (
  IN UINT8                    *User,
  IN CONST struct pcap_pkthdr *PktHdr,
  IN CONST UINT8              *Packet
  )
/*++

Routine Description:

  The callback routine of receive assertion packet

Arguments:

  User    - User string corresponds to user parameter of pcap_dispatch().
  PktHdr  - Packet header buffer.
  Packet  - Packet data buffer.

Returns:

  None

--*/
;

INT32
EmsParseAssertionPkt (
  ASSERTION_PACKAGE *AssertionPackagePtr
  )
/*++

Routine Description:

  Parse the assertion packet

Arguments:

  AssertionPackagePtr - The pointer to the assertion packet

Returns:

  -1 Failure
  0  Success

--*/
;

INT32
EmsShowAssertion (
  UINT8 *MessageBuf
  )
/*++

Routine Description:

  Show the Assertion

Arguments:

  MessageBuf  - The data of the message to show

Returns:

  0

--*/
;

VOID_P
RecvAssertionTclInit (
  IN Tcl_Interp *Interp
  )
/*++

Routine Description:

  RecvAssertion related TCL initialization.

Arguments:

  Interp      - TCL intepreter.

Returns:

  None

--*/
;

#endif
