/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsProtocols.h
    
Abstract:

    Incude header files for network protocol

--*/

#ifndef __EMS_PROTOCOLS_H__
#define __EMS_PROTOCOLS_H__

#include <tcl.h>
#include <EmsTypes.h>
#include <EmsNet.h>

/*
 * Information of each protocol field.
 * Used for packing a packet of specified protocol type.
 */
typedef struct _Field_T {
  INT8    *Name;      /* Field name */
  UINT8   Type;       /* Field type OCTET1, IPADDR, MACADDR, PAYLOAD ...*/
  VOID_P  *Value;     /* Actual value  */
  BOOLEAN Mandatory;  /* If true, this field MUST be set*/
  BOOLEAN *IsOk;      /* If this field has been set*/
} FIELD_T;

VOID_P
EmsFieldDump (
  INT8     *Buff,
  FIELD_T  *Fields
  )
/*++

Routine Description:

  Dump the field information into a specified buffer.

Arguments:

  Buff    - The buffer for the fields
  Fields  - The pointer to the fields

Returns:

  None

--*/
;

#define OCTET1  0x01
#define OCTET2  0x02
#define OCTET4  0x03
#define IPADDR  0x04
#define MACADDR 0x05
#define PAYLOAD 0x06
#define STRING  0x07

typedef
libnet_t  *
CREATE_PROC_T (
  IN INT32  Argc,
  IN INT8   *Argv[]
  );

typedef
FIELD_T   *
UNPACK_PROC_T (
  IN UINT8  *Packet,
  IN UINT32 Length
  );

typedef
VOID_P
TCL_INIT_PROC_T (
  IN Tcl_Interp *Interp
  );

typedef struct _PROTOCOL_ENTRY_T {
  INT8            *Name;          /* Protocol name */
  CREATE_PROC_T   *CreatePacket;  /* Callback routine for create packet */
  UNPACK_PROC_T   *UnpackPacket;  /* Callback routine for unpack packet */
  TCL_INIT_PROC_T *TclInit;       /* Create protocol specified tcl commands 
                                * in this routine */
} PROTOCOL_ENTRY_T;

extern PROTOCOL_ENTRY_T *Protocols[];

extern INT8             *EmsInterface;

extern
PROTOCOL_ENTRY_T        *
GetProtocolByName (
  INT8 *Name
  )
/*++

Routine Description:

  Get the protocol entry by specified name

Arguments:

  Name  - Protocl name

Returns:

  The protocol entry or NULL.

--*/
;
#endif
