/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  TslInit.h

Abstract:

  This file defines the EFI Test Support Library Initiation Interface.

  This interface provides a union format to install, uninstall, and initialize
  the test support library protocols. Using this interface the test management
  system can detect any test support libraries automatically.

--*/

#ifndef _EFI_TSL_INIT_H_
#define _EFI_TSL_INIT_H_

//
// Includes
//

#include "Efi.h"
#include "EfiTest.h"

//
// EFI TSL Initiation Interface Definitions
//

#define EFI_TSL_INIT_INTERFACE_GUID       \
  { 0x625c0828, 0xa47d, 0x493d, 0xb7, 0x97, 0x97, 0x85, 0x55, 0x72, 0xdb, 0xc8 }

#define EFI_TSL_INIT_INTERFACE_REVISION   0x00010000

//
// Forward reference for pure ANSI compatibility
//

EFI_FORWARD_DECLARATION (EFI_TSL_INIT_INTERFACE);

//
// EFI TSL Initiation Interface API - Open
//
typedef
EFI_STATUS
(EFIAPI *EFI_TSL_OPEN) (
  IN     EFI_TSL_INIT_INTERFACE         *This,
  IN OUT EFI_HANDLE                     *LibHandle,
     OUT VOID                           **PrivateInterface
  )
/*++

Routine Description:

  Opens the test support library to get the handle with the public interface
  and the private interface.

Arguments:

  This              - TSL Initiation Interface instance.

  LibHandle         - The pointer to the handle on which the public interface
                      is installed.

  PrivateInterface  - The private interface of the test support library.

Returns:

  EFI_SUCCESS if everything is correct.

--*/
;

//
// EFI TSL Initiation Interface API - Close
//
typedef
EFI_STATUS
(EFIAPI *EFI_TSL_CLOSE) (
  IN EFI_TSL_INIT_INTERFACE             *This,
  IN EFI_HANDLE                         LibHandle
  )
/*++

Routine Description:

  Closes the test support library to free the public interface.

Arguments:

  This              - TSL Initiation Interface instance.

  LibHandle         - The handle on which the public interface was installed.

Returns:

  EFI_SUCCESS if everything is correct.

--*/
;

//
// EFI TSL Initiation Interface
//

typedef struct _EFI_TSL_INIT_INTERFACE {
  UINT64                                Revision;
  EFI_GUID                              LibraryGuid;
  EFI_TSL_OPEN                          Open;
  EFI_TSL_CLOSE                         Close;
} EFI_TSL_INIT_INTERFACE;

//
// Global ID for EFI TSL Initiation Interface
//

extern EFI_GUID gEfiTslInitInterfaceGuid;

#endif
