/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCRT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  SCRTDriver.c

--*/

#include "SCRTDriver.h"


CHAR16  *gVarName             = L"UEFIRuntimeVariable";
CHAR16  *gTestRecordName      = L"TestRecord";
EFI_RUNTIME_SERVICES            *VRT;
static EFI_BOOT_SERVICES        *mBS;
static EFI_RUNTIME_SERVICES     *mRT;
static EFI_EVENT                mEfiVirtualNotifyEvent;

EFI_GUID  gRuntimeFuncAddressGuid = RUNTIME_FUNC_ADDRESS_GUID;


VOID
EFIAPI
SCRTDriverVirtualNotifyEvent (
  IN EFI_EVENT        Event,
  IN VOID             *Context
  )
{
  EFI_EVENT_NOTIFY  ChildNotify;

  if (Context != NULL) {
    ChildNotify = (EFI_EVENT_NOTIFY) (UINTN) Context;
    ChildNotify (Event, NULL);
  }
}


EFI_STATUS
SCRTRuntimeDriverInit(
  IN EFI_SYSTEM_TABLE        *SystemTable,
  IN EFI_EVENT_NOTIFY        GoVirtualChildEvent
)
{
  EFI_STATUS  Status;
  mBS  = SystemTable->BootServices;
  mRT  = SystemTable->RuntimeServices;
  VRT  = mRT;
  
  Status = mBS->CreateEvent (
                  EFI_EVENT_SIGNAL_VIRTUAL_ADDRESS_CHANGE,
                  EFI_TPL_NOTIFY,
                  SCRTDriverVirtualNotifyEvent,
                  (VOID *)(UINTN)GoVirtualChildEvent,
                  &mEfiVirtualNotifyEvent
                  );
  return Status;
}

EFI_RUNTIMESERVICE
VOID
SCRTDriverClassAddressChangeEvent (
  IN EFI_EVENT        Event,
  IN VOID             *Context
  )
/*++

Routine Description:

  The event handler to change physical address to virtual address

Arguments:

  Event   - The event that be siganlled when virtual address changed
  Context - The pointer of the procedure instance

Returns:

  None

--*/  
{
  //
  // Register a virtual notification event to convert the global pointer variables.
  //
  mRT->ConvertPointer (0, (VOID **)&VRT);
  mRT->ConvertPointer (0, (VOID **)&gVarName);
  mRT->ConvertPointer (0, (VOID **)&gTestRecordName);
}


EFI_DRIVER_ENTRY_POINT (InitializeSCRTDriver)

EFI_STATUS
InitializeSCRTDriver (
  IN EFI_HANDLE              ImageHandle,
  IN EFI_SYSTEM_TABLE        *SystemTable
  )
/*++

Routine Description:

  This function does initialization for SCRTDriver

Arguments:

  ImageHandle   - The firmware allocated handle for the EFI image.
  SystemTable   - A pointer to the EFI System Table.

Returns:
  
  Status code
  
--*/  
{
  UINTN                Size;
  UINTN                FuncAddr;
  EFI_STATUS           Status;
  SCRT_STUB_TABLE      *StubTable;
  
  //
  // Intialize Runtime Driver Lib and Register a virtual notification event.
  //
  SCRTRuntimeDriverInit (SystemTable,SCRTDriverClassAddressChangeEvent);

  FuncAddr = (UINTN) RuntimeTestFunc;

  Size = sizeof(SCRT_STUB_TABLE);
  Status  = mBS->AllocatePool (EfiRuntimeServicesData, Size, (VOID **) &StubTable);
  if (EFI_ERROR(Status)){
    return Status;
  }
  StubTable->Signature = SCRT_STUB_TABLE_SIGNATURE;
  StubTable->FuncAddr  = FuncAddr;
  
  Status = mBS->InstallConfigurationTable (&gRuntimeFuncAddressGuid, (VOID*)StubTable);
  if (EFI_ERROR(Status)){
    return Status;
  }
  return EFI_SUCCESS;
}


VOID
EFIAPI
RuntimeTestFunc (
  UINTN    HandOffAddr
  )
/*++

Routine Description:

  Perform runtime service test in a virtual addressing mode.
  
Arguments:

  NONE

Returns:

  NONE
  
--*/   
{
  CONF_INFO  ConfigData;

  //
  // Parse runtime hand off data
  //
  ConsumeHandOff(HandOffAddr, &ConfigData);
  Printf("BitMap = 0x%x\n", ConfigData.InfoData);
  if (ConfigData.BitMap.Signature != CONFIGURE_INFO_SIGNATURE) {
    Printf ("Find Configuration Data Corrupte, Hang!!\n");
    EFI_DEADLOOP();
  } 
  //
  // Dump Runtime Table
  //
  DumpRuntimeTable();

  InitVariableRecord(&ConfigData);
  Printf ("==================Variable Services Test Start==================\n\n");
  EfiVariableTestVirtual(&ConfigData);
  Printf ("===================Variable Services Test End===================\n\n\n");

  Printf ("====================Time Services Test Start====================\n\n");
  EfiTimeTestVirtual(&ConfigData);
  Printf ("=====================Time Services Test End=====================\n\n\n");

  Printf ("==================Capsule Services Test Start===================\n\n");
  EfiCapsuleTestVirtual(&ConfigData);
  Printf ("===================Capsule Services Test End====================\n\n\n");

  Printf ("====================Misc Services Test Start====================\n\n");
  EfiMiscTestVirtual(&ConfigData);
  Printf ("=====================Misc Services Test End=====================\n\n\n");

  Printf ("===================Reset Services Test Start=================\n\n");
  EfiResetTestVirtual(&ConfigData);

}
